// Copyright 2017 by Yun Xiang, Tam Chantem, and Robert Dick.
// All rights reserved.

/*Copyright 2010, Yun Xiang<xiangyun@umich.edu>.*/
#include "RStd.h"
#include "RVector.h"
#include "RGen.h"
#include "RString.h"
#include "Parse.h"
#include "FVector.h"
#include "RStd.h"
#include "RString.h"
#include "Cont.h"


#include <ctime>
#include <unistd.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

using namespace std;
using namespace rstd;


void 
PE::set_values(double ref_em_a,double ref_sm_a, 
	       double ref_tc_a, double ref_tddb_a, 
	       double ref_vol_a, double ref_tmp_a, 
		double ref_feq_a, int device_number_a, 
		double em_age_a, double sm_age_a,
	       double tc_age_a, double tddb_age_a)
{
  	ref_em_=ref_em_a;	       	
	ref_tc_=ref_tc_a;
	ref_sm_=ref_sm_a;
	ref_tddb_=ref_tddb_a;
	ref_vol_=ref_vol_a;  
	ref_tmp_=ref_tmp_a;    
	ref_feq_=ref_feq_a;	       
	em_age = em_age_a;
	tc_age = tc_age_a;
	sm_age = sm_age_a;
	tddb_age = tddb_age_a;
	device_num = device_number_a;
}

int parse(RVector<PE> & pe, const char *rel_file, RVector<string> &survive_list, RVector<RVector<RVector<double> > > &therm_profile, RVector<RVector<RVector<double> > > &time_profile, RVector<RVector<RVector<double> > > &vol_profile, RVector<RVector<RVector<double> > > &freq_profile, RVector<RVector<RVector<Cycle> > > &cycle_profile, GLOBAL &def)
{
  RVector<string> vec; 
  fstream infile;
  infile.open(rel_file, ios::in);
  if (!infile.is_open()) {
    cout<<"open file failed"<<endl;
    return 0;
  }
  string str;
  string::size_type text_pos;
  int flag = 0;
  int count = 5;
  int pe_index = 0, list_index = 0;
  while (!infile.eof()){
    getline(infile, str);
    text_pos = str.find_first_not_of(" \t");
    if (text_pos < str.size() && str[text_pos] != '#'){
      vec=tokenize(str);
      if (count > 0){
				count--;
				if((vec[0] == "l")||(vec[0] == "w")){
	  			if(vec.size() != 5){
	    			cout<<"Wrong global input format"<<endl;
	    			return 0;
	  			}
	  			def.type.push_back(vec[0]);
	  			switch(count){
	  				case 4 :
	    				for (int i = 0; i < 4; i++)
	      				def.em.push_back(Conv(vec[i+1]));
	    				break;
	  				case 3 :
	    				for (int i = 0; i < 4; i++)
	      				def.sm.push_back(Conv(vec[i+1]));
	    				break;
	  				case 2 :
	    				for (int i = 0; i < 4; i++)
	      				def.tc.push_back(Conv(vec[i+1]));
	    				break;
	  				case 1 :
	    				for (int i = 0; i < 4; i++)
	      				def.tddb.push_back(Conv(vec[i+1]));
	    				break;
	  				default: 
	    				cout<<"Wrong global input format!"<<endl;
	    				return 0;
	  			}//switch
				}//if((vec[0] == "l")
	else if (vec[0] == "t"){
	  def.tc[2] = 0;
	  def.t = Conv(vec[1]);
	}
	else {return 0;}
      }//if (count > 0)

  else if (vec[0] == "s"){
		flag = 1;
		pe_index = 0;
		list_index++;
		survive_list.push_back(vec[1]);
		time_profile.resize(list_index);
		therm_profile.resize(list_index);
		vol_profile.resize(list_index);
		freq_profile.resize(list_index);
		cycle_profile.resize(list_index);
  }
  else if ((vec[0]=="component")||(vec[0]=="c")) {
		flag = 1;
		pe_index++;
		time_profile[list_index-1].resize(pe_index);
		therm_profile[list_index-1].resize(pe_index);
		vol_profile[list_index-1].resize(pe_index);
		freq_profile[list_index-1].resize(pe_index);
		cycle_profile[list_index-1].resize(pe_index);
  }
	else if (vec[0] == "cycle") {
		flag = 2;
	}
  else {
		if (flag == 0){
	    PE pe_;
	    if (vec.size() != 12){
	      cout<<"Wrong component input format!"<<endl;
	      return 0;
	    }
	    pe_.set_values(Conv(vec[0]),Conv(vec[1]),Conv(vec[2]),Conv(vec[3]),Conv(vec[4]),Conv(vec[5]),Conv(vec[6]), Conv(vec[7]), Conv(vec[8]), Conv(vec[9]), Conv(vec[10]), Conv(vec[11]));
	    pe_.em_sigma = def.em[0];
	    pe_.sm_sigma = def.sm[0];
	    pe_.tc_sigma = def.tc[0];
	    pe_.tddb_sigma = def.tddb[0];
	    pe.push_back(pe_);
	  }
		else if (flag == 1){
	  	if (vec.size()!=4){
	    	cout<<"Wrong thermal profile format!"<<endl;
	    	cout<<"the wronged input is: "<<vec<<endl;
	    	return 0;
	  	}
	  	double a= Conv(vec[0]);
	  	a = a/(365*24*3600);
	  	time_profile[list_index-1][pe_index-1].push_back(a);
	  	therm_profile[list_index-1][pe_index-1].push_back(Conv(vec[1]));
	  	vol_profile[list_index-1][pe_index-1].push_back(Conv(vec[2]));
	  	freq_profile[list_index-1][pe_index-1].push_back(Conv(vec[3]));
		}	
		else if (flag == 2){
	  	if (vec.size()!=3){
	    	cout<<"Wrong cycle profile format!"<<endl;
	    	cout<<"the wronged input is: "<<vec<<endl;
	    	return 0;
	  	}			
			int number = Conv(vec[0]);
			Cycle tmp;
			tmp.swing = Conv(vec[1]);
			tmp.peak = Conv(vec[2]);
			tmp.index = 0; 
			for (int i = 0; i < number; i++){
				cycle_profile[list_index-1][pe_index-1].push_back(tmp);
			}
		}	
	}//else      
    }       
  }
  infile.close();
  //cout << "PE index: " << pe_index << endl;
  //cout << "core: " << therm_profile[0].size() << endl;
  //cout << "pe core num: " << pe.size() << endl;
  //cout << "Size: " << therm_profile[0][0].size() << "\t" << time_profile.size() << endl;
  return 1;
}
