// Copyright 2017 by Yun Xiang, Tam Chantem, and Robert Dick.
// All rights reserved.

/*Copyright 2010, Robert Dick<dickrp@eecs.umich.edu>.*/
#include <vector>
#include <string>
#include <unistd.h>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include "SString.h"
#include "configuration.h"

using namespace std;

#define X_ 0
#define Y_ 1
#define Z_ 2
#define DIM_C_ 3

configuration * configuration::ap_ = 0;

//*==========================================================================*/
void configuration::usage(char * const argv, const char * opt_str) {
    cerr << "Usage: " << argv << " " << opt_str << endl;
    exit(1);
}

//*==========================================================================*/
configuration::configuration(int argc, char * const argv []):
    dyn_solver_type_('t'),
	moments_(20),
    nhar_adap_(false),
	steady_coeff_(),
	pfile_()
{

	ap_ = this;
	const char * opt_str = "hl:d:r:c:f:p:y:s:aM:e:g:";
	string usage2 = string("usage: ") + argv[0] + " [" +
		opt_str + "] \n";

	while (1) {
		long ch = getopt(argc, argv, opt_str);

		if(ch == -1) {
			break;
		}

		vector<string> vec;

		switch (ch) {

			case 'h':
				help();
				break;

            case 'l':
                LUT_file_ = optarg;
                break;

            case 'r':
                rel_file_ = optarg;
                break;

            case 'd':
                device_file_ = optarg;
                break;

            case 'c':
                chip_file_ = optarg;
                break;

            case 'f':
                floorplan_file_ = optarg;
                break;

			case 'p':
				pfile_ = optarg;
				break;

            case 'y':
                layout_file_ = optarg;
                break;

            case 's':
                dyn_solver_type_ = Conv(optarg);
                break;

            case 'a':
                nhar_adap_ = true;
                break;

			case 'M':
				moments_ = Conv(optarg);
				break;

			case 'e':
				steady_coeff_.err_thre_ = Conv(optarg);
				break;

			case 'g':
				steady_coeff_.grad_thre_ = Conv(optarg);
				break;

			default :
                usage(argv[0], opt_str);
				break;
		}
	}
        
}

//*==========================================================================*/
void configuration::help() {
    string help_ =
	string("\th: help\n") +
	"\tc: chip-package file\n" +
    "\tf: floorplan file\n" +
    "\tl: lookup table file\n" +
    "\td: device file\n" +
    "\tp: power profile file\n" +
    "\ty: layout file\n" +
	"\tM: moments\n" +
	"\ts: frequency domain 'f' or time domain analysis 't' \n" +
	"\te: steady state error threshold\n" +
	"\tg: steady state gradient threshold\n";

    cout << help_ << endl;
    exit(1);
}

//*==========================================================================*/
ostream & operator<<(ostream & os, const configuration & cf) {
	os << "****************configuration********************\n" <<
        "chip file     :" << cf.chip_file_<<"\n"<<
        "floorplan file:" << cf.floorplan_file_<<"\n"<<
        "lookup table file:" << cf.LUT_file_<<"\n"<<
        "device file   :" << cf.device_file_<<"\n"<<
        "power profile file:" << cf.pfile_<<"\n"<<
        "layout file   :" << cf.layout_file_<<"\n"<<
		"moments       :" << cf.moments_<<"\n"<<
		"Dyn. solver ty:" << cf.dyn_solver_type_<<"\n"<<
		"Ste. err thre :" << cf.steady_coeff_.err_thre_<<"\n"<<
		"Ste. grad thre:" << cf.steady_coeff_.grad_thre_<<"\n"<<
		"****************configuration********************\n";
	return os;
}

